// Thanks to http://www.microchip.com/forums/m724615.aspx for trial and error;
// seems the Microchip documentation for such a simple task was lacking.
// It is also needed to look in plib.h's ports.h to find which pins actually
// support change notification (for example, RPB6 and RPB12 do not).

void __ISR( _CHANGE_NOTICE_VECTOR, IPL2SOFT) buttonState(void)
{
    // Step #1 - always clear the mismatch condition first (whatever that is))
    int tempPortRead = PORTB;
    tempPortRead = PORTA;
    
    // Step #2 - then clear the interrupt flag
    mCNBClearIntFlag();
    mCNAClearIntFlag();
    
    if (accepting_press == 0) return;     // Cheesy debounce
    accepting_press = 0;

    // Step #3 - process the switches
    // Reset peak X
    if (!mPORTBReadBits(BIT_8)){
        p_amp_x = 0;
        p_rpm_x = 0;
        p_phs_x = 0;
    } 
    // Reset peak Y
    if (!mPORTBReadBits(BIT_9)){
        p_amp_y = 0;
        p_rpm_y = 0;
        p_phs_y = 0;
    }
    // Capture null X
    if (!mPORTAReadBits(BIT_2)){
        if (x_null == 0){
            start_x_null = 1;
        } else {
            x_null = 0;
        }
    }
    // Capture null Y
    if (!mPORTBReadBits(BIT_7)){
        if (y_null == 0){
            start_y_null = 1;
        } else {
            y_null = 0;
        }
    }
}

// Button initialization function
// Set pins as input, turn on pullups, enable change notice interrupt
void initButtons(void){
    // mPORTBSetPinsDigitalIn(BIT_5);      // Capture X null
    mPORTBSetPinsDigitalIn(BIT_7);      // Capture Y null
    mPORTBSetPinsDigitalIn(BIT_8);      // X peak clear
    mPORTBSetPinsDigitalIn(BIT_9);      // Y peak clear
    
    /* STEP 1. setup the change notice options */ 
    mCNBOpen(CNB_ON | CNB_IDLE_CON, 
             CNB7_ENABLE | CNB8_ENABLE | CNB9_ENABLE,
             CNB7_PULLUP_ENABLE | CNB8_PULLUP_ENABLE | CNB9_PULLUP_ENABLE); 
    mCNAOpen(CNA_ON | CNA_IDLE_CON, CNA2_ENABLE, CNA2_PULLUP_ENABLE);
 
    /* STEP 3. read port(s) to clear mismatch */ 
    int tempPortRead = mPORTBRead();
    tempPortRead = mPORTARead();
 
    /* STEP 4. set up change notice interrupt flag */ 
    INTEnable(INT_CNB, INT_ENABLED); 
    INTEnable(INT_CNA, INT_ENABLED);
    INTSetVectorPriority(INT_CHANGE_NOTICE_VECTOR, INT_PRIORITY_LEVEL_2); 
}