/* 
 * File:   gui.h
 * Author: Anthony
 *
 * Created on October 1, 2015, 11:16 AM
 * Contains constants for drawing GUI elements
 */

#include "tft_gfx.h"
#include "tool_macros.h"

// Primary measurement configuration
// Some of these values are just "notes" - never meant to be used
#define MEASURE_F   8000
#define MIN_RPM     200
#define MAX_RPM     8000
#define MIN_VPMM    5.5
#define MAX_VPMM    8.5
#define CAL_X       1.33
#define CAL_Y       1.20
#define PEAK_JUMP   160

// GUI parameters
#define REFRESH_T   333
#define SCALE_T     200
#define HEART_N     3
#define SER_T       333
#define BKG_COL     ILI9340_BLACK
#define DIV_COL     ILI9340_WHITE
#define DIV_THK     2   // Consider this non-negotiable. A note.

// GUI parameters continued: font sizes
#define TRPM_SIZE   3
#define CHAN_SIZE   2
#define S_SIZE      1
#define STAT_SIZE   2
#define NULL_SIZE   2
#define PEAK_SIZE   2
#define DATA_SIZE   2
#define UNIT_SIZE   1

// GUI Colors
#define TRPM_COL    ILI9340_YELLOW
#define LBL_COL     ILI9340_WHITE
#define DATA_COL    ILI9340_CYAN
#define STAT_GOOD_COL ILI9340_GREEN
#define STAT_BAD_COL  ILI9340_RED
#define NULL_GOOD_COL ILI9340_GREEN
#define NULL_BAD_COL  ILI9340_RED

// GUI parameters continued: spacing and position, Y direction
#define TOP_DIV     46
#define BOT_DIV     140
#define HEART_SIZE  10
#define HEART_Y     (TOP_DIV - HEART_SIZE)/2
#define TRPM_Y       10
#define CHAN_Y      (TOP_DIV+10)
#define S_Y         (4 + CHAN_Y + 8*CHAN_SIZE)
#define NULL_Y      (BOT_DIV-10-8*NULL_SIZE)
#define STAT_Y      ((BOT_DIV+TOP_DIV)/2)
#define PEAK_Y      (BOT_DIV + 10)
#define DATA_SPACE  8
#define RPM_Y       PEAK_Y + 8*PEAK_SIZE + DATA_SPACE 
#define AMP_Y       RPM_Y + 8*DATA_SIZE + DATA_SPACE
#define PHASE_Y     AMP_Y + 8*DATA_SIZE + DATA_SPACE

// X positioning gets its own section because certain things are centered
#define HEART_X     30
#define TRPM_X      76
#define TRPM_LBL_X  (TRPM_X + 6*TRPM_SIZE*6)
#define CHANX_LBL   "X Channel"
#define CHANY_LBL   "Y Channel"
#define CHANX_X     26
#define S_LBL_W     8
#define SX_X        (80 - (S_LBL_W*4*S_SIZE)) - 16
#define SX_X_D      SX_X + ((S_LBL_W+2)*4*S_SIZE)
#define S_LBL       "Scale:     v/mm"
#define STAT_GOOD   "Measuring "
#define STAT_LOW    "Low Signal"
#define STAT_BAD    "No Signal "
#define STAT_LBL_W  10
#define STATX_X     (80 - (STAT_LBL_W*3*STAT_SIZE))
#define NULL_GOOD   "  Have Null  "
#define NULL_BAD    "No Null Vect."
#define NULL_LBL_W  13
#define NULLX_X     (80 - (NULL_LBL_W*3*NULL_SIZE))
#define PEAK_LBL    "Peak"
#define NOW_LBL     "Now"
#define PEAKX_X     14
#define NOWX_X      100
#define DATA_W      4
#define PDATAX_X    8
#define NDATAX_X    PDATAX_X + 80
#define PUNITSX_X   58
#define NUNITSX_X   PUNITSX_X + 80

// The Y channel X positions are the X channel positions shifted over
#define NUNITSY_X   160 + NUNITSX_X
#define PUNITSY_X   160 + PUNITSX_X
#define NDATAY_X    160 + NDATAX_X
#define PDATAY_X    160 + PDATAX_X
#define PEAKY_X     160 + PEAKX_X
#define NOWY_X      160 + NOWX_X
#define NULLY_X     160 + NULLX_X + 2
#define SY_X        160 + SX_X
#define SY_X_D      160 + SX_X_D
#define STATY_X     160 + STATX_X
#define CHANY_X     160 + CHANX_X